/**
  ******************************************************************************
  * @file           : main.c
  * @author         : sinomcu.com-AE
  * @brief          : Main program body
  *    
   *          This file provides example code for ADC Trigger by Timer and printf by uart.
  * ADC IN1,IN4 interval 10ms trigger by Timer1,DMA Transfer,and filter.
  * Example use time out(USE_TIMEOUT 1),when error LED1 blink period 400mS by sysTick soft
  * timer,defualt blink period 1S;uart send information of running count(redirect printf).
  * For details,see "readme.txt".  
  *         
  ******************************************************************************
   * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */  
  
#include "system_define.h"

#define LED_BLINK_HALF_PER        200            // unit ms,LED blink half cycle
#define SYSCLK_USE_PLL_48M        1              // 1:sysclk PLL48M,0:HSI 8M 

#define USER_FILTERING            16             // if more than 16,adcch1_value,adcch4_value,adc_iref_value data type need to uint32_t
#define FULL_SCALE                4095           // 12Bit

// cannot be changed
#define VDDA_3V3                  3300           // unit mV
#define AVG_SLOPE                 4.3            // unit mV/Celsius degree
#define TMP_FAB_UNIT              0.0625         // unit Celsius degree

FlagStatus DMA_CH1_TC_Trigger = RESET;
uint16_t ADC_result_buff[ADC_BUFF_LEN]={0};

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void) {
  ADCErrorStatus ADC_status = ADC_NO_ERROR;
  uint8_t filter_count=0;
  uint16_t vrefint_cal = 0,adcch1_value=0,adcch4_value=0,adc_iref_value=0,vdda_mv=0;
  uint32_t vdda_cal=0;
  uint16_t adc_fab_value=0,Tmp_fab_num=0;        // datasheet ADC_Temp1,Sens_Temp1
  int32_t adc_tmp_value = 0;
  float Tmp_fab_offset = 0,temperature = 0;

#if(defined(SYSCLK_USE_PLL_48M) && SYSCLK_USE_PLL_48M) 
  sysclk_init();                            // sysclk 48M,HSI 8M /2*12;if not call this function,default HSI 8M
#endif
  SystemCoreClockUpdate();                  // get System Core Clock

  sys_tick_init();
  gpio_init();
#if(defined(EN_PRINT_UART) && EN_PRINT_UART)   
  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_HSI);  // uart clk HSI 8M      
  usart1_uart_init(115200);
#endif
  dma_ch1adc_init();
  adc_init();
  timer_init();

  LED1_ON(); 
  printf("\r\n\r\n*****ADC DMA (AIN0,AIN4,Iref,Tmp) TimerTrig Example*****");
  printf("\r\n--Inf:System Core Clock %d Hz.",SystemCoreClock);
  
  ADC_status = Activate_ADC();
  if(ADC_status !=ADC_NO_ERROR ) {
    ADC_ERROR(ADC_status);
  }

  MS32_ADC_REG_StartConversion(ADC1);                           // Hardware Trigger need 

  vrefint_cal    = MS32_VREFINT();
  // adc_fab_value and Tmp_fab_num are factory calibrated values,see datesheet 3.20.1 description
  adc_fab_value  = MS32_GetV30();                               // get ADC_Temp1
  Tmp_fab_num    = MS32_GetTmp30();                             // get Sens_Temp1
  Tmp_fab_offset = Tmp_fab_num *TMP_FAB_UNIT;                   // calc Tmp_offset

  Start_User_MS_SF_Timer(LED_BLINK_HALF_PER);
  while(1) {
    if(DMA_CH1_TC_Trigger == SET) {                             // DMA interrupt callback set flag,DMA transfer one time finish
      DMA_CH1_TC_Trigger = RESET;
      adc_iref_value += ADC_result_buff[3];
      adcch1_value   += ADC_result_buff[0]; 
      adcch4_value   += ADC_result_buff[1];
      adc_tmp_value  +=ADC_result_buff[2];
      filter_count++;

      if(filter_count == USER_FILTERING){
        filter_count = 0;
        adc_iref_value = adc_iref_value / USER_FILTERING;
        adcch1_value   = adcch1_value   / USER_FILTERING;   
        adcch4_value   = adcch4_value   / USER_FILTERING; 
        adc_tmp_value  = adc_tmp_value  / USER_FILTERING;

        // VDDA = 3.3V x VREFINT_CAL / VREFINT_DATA
        vdda_cal = (VDDA_3V3 * vrefint_cal) /adc_iref_value;
        vdda_mv  = (uint16_t)vdda_cal;
        printf("\r\n\r\n--Inf:VDDA on board is %dmv.", vdda_mv);

        printf("\r\n--Inf:ADC value ch1 %d,ch4 %d;Voltage ch1 %dmV,ch4 %dmV,.", adcch1_value,adcch4_value, \
                    __MS32_ADC_CALC_DATA_TO_VOLTAGE(vdda_mv, adcch1_value, MS32_ADC_RESOLUTION_12B), \
                    __MS32_ADC_CALC_DATA_TO_VOLTAGE(vdda_mv, adcch4_value, MS32_ADC_RESOLUTION_12B));
        
        // user manual 12.9      :temperature = (    Vsense                   -       V30           ) / Avg_Slope + Tmp30
        // datasheet,ADC 12Bit,mV:temperature = (adc_tmp_value*USER_VDDA/4095 - ADC_Temp1*3300/4095 ) / Avg_Slope + Tmp_offset
        // program  ,ADC 12Bit,mV:temperature = (adc_tmp_value*USER_VDDA      - adc_fab_value*3300  )/4095 /Avg_Slope +Tmp_fab_offset
        temperature = (float)(adc_tmp_value *vdda_mv  - adc_fab_value*VDDA_3V3)/ FULL_SCALE / AVG_SLOPE + Tmp_fab_offset;  // calc temperature
        printf("\r\n--Inf:adc_value is %d,temperature is %2.2f .", adc_tmp_value,temperature);

        adc_iref_value = 0;
        adcch1_value   = 0;
        adcch4_value   = 0;
        adc_tmp_value  = 0;
      }
    }

    if(MS32_ADC_IsActiveFlag_OVR(ADC1) ==SET) {
      ADC_ERROR(ADC_OVERRUN_ERROR);
    }
    if(Get_State_User_MS_SF_Timer() == SF_TIMER_OV) {
      Start_User_MS_SF_Timer(LED_BLINK_HALF_PER);               // clear SF_TIMER_OV
      LED1_TOGGLE();
    }
  }
}

/**
  * @brief  SystemClock_Config HSI/2 to PLL*12 sysclk 48M
  * @param  None
  * @retval None
  */
void sysclk_init(void) {
  MS32_RCC_OscInitTypeDef osc_cfg;
  MS32_RCC_PllInitTypeDef pll_cfg;
  MS32_RCC_ClkInitTypeDef clk_cfg;

  // get and clear reset flag 
  MS32_RCC_GetResetFlag(ENABLE);
  
  // osc config 
  osc_cfg.HSIState = MS32_RCC_HSI_ON;
  osc_cfg.HSEState = MS32_RCC_HSE_OFF;
  osc_cfg.LSIState = MS32_RCC_LSI_OFF;
  osc_cfg.LSEState = MS32_RCC_LSE_OFF;                     // LSE off
  MS32_RCC_OscConfig(&osc_cfg);
  
  // pll config 
  pll_cfg.PLLState  = MS32_RCC_PLL_ON;
  pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSI_DIV_2;            // HSI/2 4M
  pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL12;
  pll_cfg.PREDIV    = MS32_RCC_PLL_PREDIV_DIV1;
  MS32_RCC_PllConfig(&pll_cfg);
  
  // sysclk config 
  clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_PLL;
  clk_cfg.AHBCLKDivider  = MS32_RCC_SYSCLK_DIV1;
  clk_cfg.APB1CLKDivider = MS32_RCC_HCLK_DIV1;
  MS32_RCC_ClockConfig(&clk_cfg);
}

/******************************** END OF FILE *********************************/
